"use strict";
this.name	= "separatedlasers";
this.author	= "Norby";
this.copyright	= "2016 Norby";
this.description= "Lasers will not heat up faster if there is at least 30m space between multiple mounts";
this.licence	= "CC BY-NC-SA 4.0";


//worldscript events
this.startUpComplete = this.playerBoughtNewShip = function() {
    this._Replace_Lasers(player.ship, this); //check and replace lasers to separated if possible
}

this.shipSpawned = function(ship) {
    this._Replace_Lasers(ship, this); //check and replace NPC lasers to separated if possible
}

this.playerBoughtEquipment = function( equipment ) {
    if( equipment.indexOf("EQ_WEAPON_") > -1 ) { //core lasers, laser cannons and other lasers in the future
        //check and replace new laser to separated
        //and replace back to basic laser if bought to not separated side
        this._Replace_Lasers(player.ship, this);
    }
}

//condition script
this.allowAwardEquipment = function(equipment, ship, context)
{
        // OXP hook to allow stations to forbid specific equipment
        if (context == "purchase" && player.ship.dockedStation && player.ship.dockedStation.scriptInfo["oolite-barred-equipment"])
        {
                if (player.ship.dockedStation.scriptInfo["oolite-barred-equipment"].indexOf(equipment) != -1)
                {
                        return false;
                }
        }

        // OXP hook to allow ships to forbid specific "available to all" equipment
        if (ship.scriptInfo && ship.scriptInfo["oolite-barred-equipment"] && ship.scriptInfo["oolite-barred-equipment"].indexOf(equipment) != -1)
        {
                return false;
        }

//      player.consoleMessage( eqKey+" "+ship+" "+context );//debug

        //check for multiple mounts
        var m = this._getMultiLevel(equipment);
        //chech for a proper multiple mount on this ship
        if( m > 0 && ship.weaponPositionForward.length != m
                && ship.weaponPositionAft.length != m
                && ship.weaponPositionPort.length != m
                && ship.weaponPositionStarboard.length != m ) {

                if( m < 5 ) return false; //dual, triple and quad lasers need exact match in mounts

                if( ship.weaponPositionForward.length < m //allow broadside lasers at and over 5 mounts
                        && ship.weaponPositionAft.length < m
                        && ship.weaponPositionPort.length < m
                        && ship.weaponPositionStarboard.length < m )
                        return false;
        }

        //Sniper Gun is checked in its own script (need specific subEntity on the ship)
        if( equipment.indexOf("EQ_WEAPON_CANNON_SNIPER_GUN") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_HEAVY_SNIPER_GUN") > -1 ) {
                var w = worldScripts.snipergun;
                if( !w ) return false;
                return( w._allowAwardSniperGun(equipment, ship, context, w) );
        }

        //Eco Artillery need as large ship as an Anaconda
        if( ship.mass < 400000 && equipment.indexOf("EQ_WEAPON_CANNON_EA") > -1 ) return false;

        //Eco Reactor, Vulcan Cannon and Plasma Web need as large ship as a Python
        if( ship.mass < 250000 &&
                (  equipment.indexOf("EQ_WEAPON_ECO_REACTOR") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_7_") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_9_") > -1 ) ) return false;

        //Eco Laser, Mini Cannon and Mini Plasma Ray fit into escort ships from 30t
        if( ship.mass <= 30000 &&
                (  equipment.indexOf("EQ_WEAPON_ECO_LASER") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_2_") > -1
                || equipment.indexOf("EQ_WEAPON_CANNON_8_") > -1 ) ) return false;

        //Other cannons for large ships only like Cobra Mark III
        if( equipment.indexOf("EQ_WEAPON_CANNON") > -1 && ship.mass < 130000 ) return false;

        return true;
}


//internal methods
this._Replace_Lasers = function(ship, w) {  //check and replace lasers to separated if possible
    if( !w ) var w = worldScripts.separatedlasers;
    var a = w._Separated_Lasers(ship, w);
    if( a ) {
        log(w.name, ship.name+" Separated laser distances: fw:"+a[1]+" aft:"+a[2]+" port:"+a[3]+" sb:"+a[4]);

        var p = player.ship;
//      log(w.name, equipmentKey+" "+datakey+" "+ship.forwardWeapon+" "+ship.aftWeapon+" "+ship.portWeapon+" "+ship.starboardWeapon); //debug

        //replace back to single laser if bought a separated one to not separated side
        //then replace single lasers to separated versions where there is enough space and money
        //use the laser match to the number of mounts but max.5
        //Dual, Triple, Quad or Broad Separated Pulse, Beam or Military Lasers, Sniper Guns and some Laser Cannons

        if( ship.forwardWeapon ) {
            var eq = ship.forwardWeapon.equipmentKey;
            var len = ship.weaponPositionForward.length;
            //replace back to basic laser if bought to not separated side
            var m = w._getMultiLevel(eq);
            if( m > 1 && ( Math.min(5, len) != m || !a[1] || a[1] < w._space(eq) ) ) {
                var eq1 = eq.substr(0, eq.length-2); //replace to single weapon without _2, _3, _4 or _5 at the end
                if( !EquipmentInfo.infoForKey( eq1 ) ) eq1 = eq; //for sure
                if( ship == p ) w._refund( eq, eq1, len, true );
                ship.forwardWeapon = eq1;
                eq = eq1;
                m = 1;
            }
            //replace single lasers to separated versions where there is enough space and money
            if( m < 2 && len > 1 && a[1] && a[1] >= w._space(eq) ) {
                var eq1 = eq+"_"+Math.min(5, len); //max 5 (broadside)
                if( EquipmentInfo.infoForKey( eq1 ) ) { //not all weapons are separable
                    if( ship == p ) { //player
                        if( w._refund( eq, eq1, len, true ) )
                            ship.forwardWeapon = eq1;
                    } else ship.forwardWeapon = eq1;
                }
            }
        }

        if( ship.aftWeapon ) {
            var eq = ship.aftWeapon.equipmentKey;
            var len = ship.weaponPositionAft.length;
            //replace back to basic laser if bought to not separated side
            var m = w._getMultiLevel(eq);
            if( m > 1 && ( Math.min(5, len) != m || !a[2] || a[2] < w._space(eq) ) ) {
                var eq1 = eq.substr(0, eq.length-2); //replace to single weapon without _2, _3, _4 or _5 at the end
                if( !EquipmentInfo.infoForKey( eq1 ) ) eq1 = eq; //for sure
                if( ship == p ) w._refund( eq, eq1, len, true );
                ship.aftWeapon = eq1;
                eq = eq1;
                m = 1;
            }
            //replace single lasers to separated versions where there is enough space and money
            if( m < 2 && len > 1 && a[2] && a[2] >= w._space(eq) ) {
                var eq1 = eq+"_"+Math.min(5, len); //max 5 (broadside)
                if( EquipmentInfo.infoForKey( eq1 ) ) { //not all weapons are separable
                    if( ship == p ) { //player
                        if( w._refund( eq, eq1, len, true ) )
                            ship.aftWeapon = eq1;
                    } else ship.aftWeapon = eq1;
                }
            }
        }

        if( ship.portWeapon ) {
            var eq = ship.portWeapon.equipmentKey;
            var len = ship.weaponPositionPort.length;
            //replace back to basic laser if bought to not separated side
            var m = w._getMultiLevel(eq);
            if( m > 1 && ( Math.min(5, len) != m || !a[3] || a[3] < w._space(eq) ) ) {
                var eq1 = eq.substr(0, eq.length-2); //replace to single weapon without _2, _3, _4 or _5 at the end
                if( !EquipmentInfo.infoForKey( eq1 ) ) eq1 = eq; //for sure
                if( ship == p ) w._refund( eq, eq1, len, true );
                ship.portWeapon = eq1;
                eq = eq1;
                m = 1;
            }
            //replace single lasers to separated versions where there is enough space and money
            if( m < 2 && len > 1 && a[3] && a[3] >= w._space(eq) ) {
                var eq1 = eq+"_"+Math.min(5, len); //max 5 (broadside)
                if( EquipmentInfo.infoForKey( eq1 ) ) { //not all weapons are separable
                    if( ship == p ) { //player
                        if( w._refund( eq, eq1, len, true ) )
                            ship.portWeapon = eq1;
                    } else ship.portWeapon = eq1;
                }
            }
        }

        if( ship.starboardWeapon ) {
            var eq = ship.starboardWeapon.equipmentKey;
            var len = ship.weaponPositionStarboard.length;
            //replace back to basic laser if bought to not separated side
            var m = w._getMultiLevel(eq);
            if( m > 1 && ( Math.min(5, len) != m || !a[4] || a[4] < w._space(eq) ) ) {
                var eq1 = eq.substr(0, eq.length-2); //replace to single weapon without _2, _3, _4 or _5 at the end
                if( !EquipmentInfo.infoForKey( eq1 ) ) eq1 = eq; //for sure
                if( ship == p ) w._refund( eq, eq1, len, true );
                ship.starboardWeapon = eq1;
                eq = eq1;
                m = 1;
            }
            //replace single lasers to separated versions where there is enough space and money
            if( m < 2 && len > 1 && a[4] && a[4] >= w._space(eq) ) {
                var eq1 = eq+"_"+Math.min(5, len); //max 5 (broadside)
                if( EquipmentInfo.infoForKey( eq1 ) ) { //not all weapons are separable
                    if( ship == p ) { //player
                        if( w._refund( eq, eq1, len, true ) )
                            ship.starboardWeapon = eq1;
                    } else ship.starboardWeapon = eq1;
                }
            }
        }

    }
}

this._getMultiLevel = function(equipment) { //get the multiplication level of a given weapon
        var m = 0;
        var end = equipment.substr(-2, 2); //the last two characters of equipmentKey
        if( end == "_2" && equipment != "EQ_WEAPON_CANNON_2" ) m = 2; //except single Ingram 3838C8 Mini Cannon
        if( end == "_3" && equipment != "EQ_WEAPON_CANNON_3" ) m = 3; //except single Hassoni C38 Dual Cannon
        if( end == "_4" && equipment != "EQ_WEAPON_CANNON_4" ) m = 4; //except single Lance & Ferman Military Cannon
        if( end == "_5" && equipment != "EQ_WEAPON_CANNON_5" ) m = 5; //except single LF38 Twin Military Cannon
        return( m );
}

this._Separated_Lasers = function(ship, w) {  //return false if no separated mount, else return array of sides
    if( !w ) var w = worldScripts.separatedlasers;
    var a = [];
    a[1] = w._is_Separated_Mount(ship, 1, w); //forward
    a[2] = w._is_Separated_Mount(ship, 2, w); //aft
    a[3] = w._is_Separated_Mount(ship, 3, w); //port
    a[4] = w._is_Separated_Mount(ship, 4, w); //starboard
//  log(w.name, ship.name+" Laser distances: fw:"+a[1]+" aft:"+a[2]+" port:"+a[3]+" sb:"+a[4]);
    if( !a[1] && !a[2] && !a[3] && !a[4] ) return false;
    else return a; //array of min. distances at sides where lasers are separated, false at other sides
}

this._is_Separated_Mount = function(ship, side, w) { //return false if this side is not separated
    if( !w ) var w = worldScripts.separatedlasers;
    var d = w._mount_Distance(ship, side, w);
    if( d < 30 ) return false;
    else return d; //return min. distance if more than the minimally allowed for separated lasers
}

this._mount_Distance = function(ship, side, w) { //return 0 if only one mount, min. dist. if more.
    if( !w ) var w = worldScripts.separatedlasers;
    side = Math.floor(side);
    var array;
    if( !side || side <= 1 || side > 4 ) array = ship.weaponPositionForward;
    else if( side == 2 ) array = ship.weaponPositionAft;
    else if( side == 3 ) array = ship.weaponPositionPort;
    else array = ship.weaponPositionStarboard;
    var min = w._min_Distance(array);
    return min;
}

this._min_Distance = function(array) { //calculate min. dist. in the given array of Vector3Ds
    var min = 0;
    if( array && array.length > 1 ) for(var i = 0; i < array.length; i++) {
      for(var j = i + 1; j < array.length; j++) { //check against all remaining items
        var d = 0;
        if( array[i].distanceTo && array[j].distanceTo ) d = array[i].distanceTo(array[j]);
        if( d > 0 && min == 0 ) min = d;
        else if( min > 0 && d < min ) min = d;
//      log(this.name, "d:"+d+"min:"+min+" a["+i+"]: "+array[i]+" a["+j+"]:"+array[j]);
      }
    }
//  log(this.name, " array: "+array+" min.d:"+min);
    return min;
}

this._refund = function(eq, eq1, length, msg) { //refund weapon price, length is the size of multiple weapons array
        var f = 1;
        if( player.ship && player.ship.dockedStation ) {
                var e = player.ship.dockedStation.equipmentPriceFactor;
                if( e > 0 ) f = e;
        }
        if( 0 < oolite.compareVersion("1.83") ) length = 1; //no multiple weapons before Oolite 1.83

        var refund = f * length * (EquipmentInfo.infoForKey( eq ).price - EquipmentInfo.infoForKey( eq1 ).price)/10;
        if( refund < 0 && player.credits < -1 * refund ) {
            var m = EquipmentInfo.infoForKey( eq ).name + " is not the best in this mount, but you have not "
                + (-1*refund) + " credits to buy " + EquipmentInfo.infoForKey( eq1 ).name+".";
            if(msg) player.consoleMessage(m, 10);
            log(this.name, "Replace "+eq+" to "+eq1+" skipped, need "+refund+"cr but you have "+player.credits+"cr only.");
            return( false );
        }
        player.credits += refund;
        var r = ", "+refund+" credits refunded.";
        if( refund < 0 ) r = " for " + (-1*refund) + " credits.";
        var m = EquipmentInfo.infoForKey( eq ).name + " is not the best in this mount, replaced to "
                + EquipmentInfo.infoForKey( eq1 ).name + r;
        if(msg) player.consoleMessage(m, 10);
        log(this.name, "Replace "+eq+" to "+eq1+", refund: "+refund+" cr");
        return( true );
}

this._space = function(eq) { //how much space need between weapon positions to be separated

        if( eq.indexOf("EQ_WEAPON_CANNON_10") > -1 //Plasma Ray
                || eq.indexOf("EQ_WEAPON_CANNON_EA") > -1 //Eco Artillery
                || eq.indexOf("EQ_WEAPON_ECO_REACTOR") > -1 //Eco Reactor
                || eq.indexOf("EQ_WEAPON_CANNON_SNIPER_GUN") > -1 ) //Sniper Gun
                return(50); //need 50m

        if( eq.indexOf("EQ_WEAPON_CANNON_HEAVY_SNIPER_GUN") > -1 ) //Heavy Sniper Gun
                return(100); //need 100m

        return(30); //all others need 30m
}
